
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity ALU is
port (
  A, B : in  STD_LOGIC_VECTOR(7 downto 0); 
  CTL  : in  STD_LOGIC_VECTOR(2 downto 0);                  
  Y    : out STD_LOGIC_VECTOR(7 downto 0) );
end ALU;

architecture ALU_ARCH of ALU is

  component ADDSUB_8
  port (
    A, B : in  STD_LOGIC_VECTOR(7 downto 0); 
    SEL  : in  STD_LOGIC;                    
    Y    : out STD_LOGIC_VECTOR(7 downto 0); 
    C    : out STD_LOGIC );
  end component;

  component AND_2_8 
  port( 
    A : IN  STD_LOGIC_VECTOR( 7 downto 0 );
    B : IN  STD_LOGIC_VECTOR( 7 downto 0 );
    Z : OUT STD_LOGIC_VECTOR( 7 downto 0 ) );
  end component;

  component MUX_2_8 
  port (
    A, B : in  STD_LOGIC_VECTOR(7 downto 0); 
    SEL  : in  STD_LOGIC;                    
    Y    : out STD_LOGIC_VECTOR(7 downto 0) );
  end component; 

  signal mux_int : STD_LOGIC_VECTOR(7 downto 0); 
  signal addsub_int : STD_LOGIC_VECTOR(7 downto 0); 
  signal and_int : STD_LOGIC_VECTOR(7 downto 0);
 
  signal carry : STD_LOGIC; 

begin

 mux_a : MUX_2_8 port map(
    A   => mux_int,
    B   => B,
    SEL => CTL(2),                
    Y   => Y );

 muxb : MUX_2_8 port map(
    A   => addsub_int,
    B   => and_int,
    SEL => CTL(1),                
    Y   => mux_int );

  adder : ADDSUB_8 port map(
    A   => A, 
    B   => B,
    SEL => CTL(0),                  
    Y   => addsub_int,
    C   => carry );

  bitwiseAND : AND_2_8 port map( 
    A => A,
    B => B,
    Z => and_int );

end ALU_ARCH;