
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity computer is
port(
  CLK : in STD_LOGIC;
  CLR : in STD_LOGIC;
  GPI : in  STD_LOGIC_VECTOR(7 downto 0); 
  GPO : out STD_LOGIC_VECTOR(7 downto 0) );
end computer;

architecture computer_arch of computer is
  
  component clock_divider_1
  port(
    CLKIN_IN        : IN std_logic;
    RST_IN          : IN std_logic;          
    CLKDV_OUT       : OUT std_logic;
    CLKIN_IBUFG_OUT : OUT std_logic;
    CLK0_OUT        : OUT std_logic;
    LOCKED_OUT      : OUT std_logic );
  end component;

  component async_reset 
  port( 
    clk         : in  STD_LOGIC;
    locked      : in  STD_LOGIC;
    reset_in    : in  STD_LOGIC;
    reset_out   : out STD_LOGIC;
    reset_out_n : out STD_LOGIC );
  end component;
  
  component simplecpu_v1a 
  port(
    CLK    : in  STD_LOGIC;
    CLR    : in  STD_LOGIC;
    ROM_EN : out STD_LOGIC;   
    RAM_EN : out STD_LOGIC;
    RAM_WR : out STD_LOGIC;
    ADDR   : out STD_LOGIC_VECTOR(7 downto 0); 
    DIN    : in  STD_LOGIC_VECTOR(15 downto 0); 
    DOUT   : out STD_LOGIC_VECTOR(15 downto 0) ); 
  end component;
  
  component RAM 
  port (
    CLK  : in  STD_LOGIC;         
    WE   : in  STD_LOGIC;                
    ADDR : in  STD_LOGIC_VECTOR(7 downto 0);
    DIN  : in  STD_LOGIC_VECTOR(15 downto 0); 
    DOUT : out STD_LOGIC_VECTOR(15 downto 0) );
  end component;
  
  component GPIO_8 
  port (
    CLK  : in  STD_LOGIC; 
    CLR  : in  STD_LOGIC; 
    WE   : in  STD_LOGIC; 
    CE   : in  STD_LOGIC; 
    ADDR : in  STD_LOGIC;
    DIN  : in  STD_LOGIC_VECTOR(15 downto 0); 
    DOUT : out STD_LOGIC_VECTOR(15 downto 0);
    GPI  : in  STD_LOGIC_VECTOR(7 downto 0); 
    GPO  : out STD_LOGIC_VECTOR(7 downto 0) );
  end component;
  
  component MUX_2_16 
  port (
    A, B : IN  STD_LOGIC_VECTOR(15 downto 0); 
    SEL  : IN  STD_LOGIC;                    
    Y    : OUT STD_LOGIC_VECTOR(15 downto 0));
  end component;
  
  component AND_7 
  port(
    A : IN STD_LOGIC_VECTOR( 6 downto 0 );
    Z : OUT STD_LOGIC);
  end component;
  
  signal clk_int   : STD_LOGIC;
  signal clr_int   : STD_LOGIC;
  signal clr_n_int : STD_LOGIC;  
  signal locked    : STD_LOGIC;
  signal clk_int_bufg : STD_LOGIC; 
  signal clk_int_50MHz : STD_LOGIC;
  
  signal addr     : STD_LOGIC_VECTOR(7 downto 0);
  signal data_in  : STD_LOGIC_VECTOR(15 downto 0);
  signal data_out : STD_LOGIC_VECTOR(15 downto 0);
  
  signal mem_data_out : STD_LOGIC_VECTOR(15 downto 0);
  
  signal gpio_data_out : STD_LOGIC_VECTOR(15 downto 0);
  signal gpio_cs   : STD_LOGIC;

  signal ram_wr   : STD_LOGIC;
  signal row_en   : STD_LOGIC; 
  signal ram_en   : STD_LOGIC;
  
begin

  clock : clock_divider_1 port map(
    CLKIN_IN        => CLK,
    RST_IN          => CLR,         
    CLKDV_OUT       => clk_int,
    CLKIN_IBUFG_OUT => clk_int_bufg,
    CLK0_OUT        => clk_int_50MHz,
    LOCKED_OUT      => locked ); 
  
  reset : async_reset port map( 
    clk         => clk_int,        
    locked      => locked,   
    reset_in    => CLR,
    reset_out   => clr_int, 
    reset_out_n => clr_n_int );

  cpu : simplecpu_v1a port map(
    CLK    => clk_int,    
    CLR    => clr_int,
    ROM_EN => row_en, 
    RAM_EN => ram_en,
    RAM_WR => ram_wr,
    ADDR   => addr,
    DIN    => data_in,
    DOUT   => data_out ); 
  
  mem : RAM port map(
    CLK  => clk_int,      
    WE   => ram_wr,            
    ADDR => addr,
    DIN  => data_out,
    DOUT => mem_data_out );
	 
  gpio : GPIO_8 port map(
    CLK  => clk_int,
    CLR  => clr_int, 
    WE   => ram_wr,               
    CE   => gpio_cs,
    ADDR => addr(0),
    DIN  => data_out,
    DOUT => gpio_data_out,
    GPI  => GPI,  
    GPO  => GPO );
	 
  addr_decoder : AND_7 port map(
    A => addr(7 downto 1),
    Z => gpio_cs );	 
 
  mux : MUX_2_16 port map(
    A   => mem_data_out,
	 B   => gpio_data_out,
    SEL => gpio_cs,               
    Y   => data_in );
	 
end computer_arch;

