
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;

use IEEE.STD_LOGIC_TEXTIO.ALL;
use std.textio.ALL;

ENTITY computer_tb IS
END computer_tb;
 
ARCHITECTURE behavior OF computer_tb IS 
 
    COMPONENT computer
    PORT(
      CLK : in STD_LOGIC;
      CLR : in STD_LOGIC;
      GPI : in  STD_LOGIC_VECTOR(7 downto 0); 
      GPO : out STD_LOGIC_VECTOR(7 downto 0) );		
    END COMPONENT;

   signal CLK : std_logic := '0';
   signal CLR : std_logic := '0';

   signal GPI : std_logic_vector(7 downto 0);			
   signal GPO : std_logic_vector(7 downto 0);

   constant CLK_period : time := 10 ns;
	
    function to_hex_string(vec: std_logic_vector) return string is
        variable hex_string : string(1 to (vec'length / 4));
        variable nibble     : std_logic_vector(3 downto 0);
    begin
        for i in 0 to (vec'length / 4 - 1) loop
            nibble := vec(vec'length - 1 - i * 4 downto vec'length - 4 - i * 4);
            case nibble is
                when "0000" => hex_string(i + 1) := '0';
                when "0001" => hex_string(i + 1) := '1';
                when "0010" => hex_string(i + 1) := '2';
                when "0011" => hex_string(i + 1) := '3';
                when "0100" => hex_string(i + 1) := '4';
                when "0101" => hex_string(i + 1) := '5';
                when "0110" => hex_string(i + 1) := '6';
                when "0111" => hex_string(i + 1) := '7';
                when "1000" => hex_string(i + 1) := '8';
                when "1001" => hex_string(i + 1) := '9';
                when "1010" => hex_string(i + 1) := 'A';
                when "1011" => hex_string(i + 1) := 'B';
                when "1100" => hex_string(i + 1) := 'C';
                when "1101" => hex_string(i + 1) := 'D';
                when "1110" => hex_string(i + 1) := 'E';
                when "1111" => hex_string(i + 1) := 'F';
                when others => hex_string(i + 1) := 'X'; -- Invalid nibble
            end case;
        end loop;
        return hex_string;
    end to_hex_string;

BEGIN

   uut: computer PORT MAP (
     CLK => CLK,
     CLR => CLR,
	  GPI => GPI,
	  GPO => GPO );

   CLK_process :process
   begin
		CLK <= '0';
		wait for CLK_period/2;
		CLK <= '1';
		wait for CLK_period/2;
   end process;
 
   CLR_process :process
   begin
		CLR <= '1';
		wait for CLK_period * 3;
		CLR <= '0';
		wait;
   end process;
	
   GPI_process :process
   begin
		GPI <= "10101010";
		wait;
   end process;
	
   debug: process
     variable data : line;
   begin
     wait until GPO'event; 
	  
     write(data, now);
     write(data, string'(" : GPO = "));
     write(data, GPO);
     write(data, string'(" = "));
     write(data, to_hex_string(GPO));
     writeline(output, data); 
   end process;

END;
