
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity CONTROL_LOGIC is
port (
  CLK      : in  STD_LOGIC;                 
  CLR      : in  STD_LOGIC;                
  D        : in  STD_LOGIC_VECTOR(3 downto 0);    
  Z        : in  STD_LOGIC; 
  ROM_EN   : OUT STD_LOGIC;  
  RAM_EN   : OUT STD_LOGIC;  
  RAM_WR   : OUT STD_LOGIC;   
  ADDR_SEL : OUT STD_LOGIC;  
  DATA_SEL : OUT STD_LOGIC; 
  ALU_CTL0 : OUT STD_LOGIC; 
  ALU_CTL1 : OUT STD_LOGIC;  
  ALU_CTL2 : OUT STD_LOGIC;  
  ACC_EN   : OUT STD_LOGIC;  
  IR_EN    : OUT STD_LOGIC; 
  PC_LD    : OUT STD_LOGIC;  
  PC_EN    : OUT STD_LOGIC );  
end CONTROL_LOGIC;

architecture CONTROL_LOGIC_ARCH of CONTROL_LOGIC is

  component RING_COUNTER_3 
  port (
    CLK : in STD_LOGIC;                 
    RST : in STD_LOGIC;           
    Q   : out STD_LOGIC_VECTOR(2 downto 0));
  end component;
  
  component ONEHOT_DECODER_16 
  port (
    A : in  STD_LOGIC_VECTOR(3 downto 0); 
    Y : out STD_LOGIC_VECTOR(15 downto 0) );
  end component;
	
  component DECODER 
  port (
    FETCH       : in  STD_LOGIC;                 
    DECODE      : in  STD_LOGIC;                
    EXECUTE     : in  STD_LOGIC;   
    MOVE        : in  STD_LOGIC; 
    ADD         : in  STD_LOGIC;  
    SUB         : in  STD_LOGIC; 
    BITWISE_AND : in  STD_LOGIC; 
    LOAD        : in  STD_LOGIC; 
    ADDM        : in  STD_LOGIC; 
    SUBM        : in  STD_LOGIC; 
    STORE       : in  STD_LOGIC; 
    JUMPU       : in  STD_LOGIC; 
    JUMPZ       : in  STD_LOGIC; 
    JUMPNZ      : in  STD_LOGIC;  
    Z           : in  STD_LOGIC; 
    ROM_EN      : OUT STD_LOGIC;  
    RAM_EN      : OUT STD_LOGIC;  
    RAM_WR      : OUT STD_LOGIC;   
    ADDR_SEL    : OUT STD_LOGIC;  
    DATA_SEL    : OUT STD_LOGIC; 
    ALU_CTL0    : OUT STD_LOGIC; 
    ALU_CTL1    : OUT STD_LOGIC;  
    ALU_CTL2    : OUT STD_LOGIC;  
    ACC_EN      : OUT STD_LOGIC;  
    IR_EN       : OUT STD_LOGIC; 
    PC_LD       : OUT STD_LOGIC;  
    PC_EN       : OUT STD_LOGIC );
  end component;
 
  signal instr       : STD_LOGIC_VECTOR(15 downto 0);
  signal move        : STD_LOGIC;
  signal add         : STD_LOGIC;  
  signal sub         : STD_LOGIC;
  signal bitwise_and : STD_LOGIC;
  signal load        : STD_LOGIC;
  signal store       : STD_LOGIC;
  signal addm        : STD_LOGIC;
  signal subm        : STD_LOGIC;
  signal jumpu       : STD_LOGIC;
  signal jumpz       : STD_LOGIC;
  signal jumpnz      : STD_LOGIC;
  
  signal phase   : STD_LOGIC_VECTOR(2 downto 0);
  signal fetch   : STD_LOGIC;
  signal decode  : STD_LOGIC;
  signal execute : STD_LOGIC;  
  
begin

  fetch   <= phase(0);
  decode  <= phase(1); 
  execute <= phase(2);
  
  move        <= instr(0);
  add         <= instr(1);
  sub         <= instr(2);
  bitwise_and <= instr(3);
  load        <= instr(4);
  store       <= instr(5);
  addm        <= instr(6);
  subm        <= instr(7);
  jumpu       <= instr(8);
  jumpz       <= instr(9);
  jumpnz      <= instr(10); 
  
  state : RING_COUNTER_3 port map(
    CLK => CLK,              
    RST => CLR,
    Q   => phase );
	 
  instr_decoder : ONEHOT_DECODER_16 port map(
    A => D, 
    Y => instr );
  
  output_logic : DECODER port map(
    FETCH       => fetch,                
    DECODE      => decode,               
    EXECUTE     => execute,  
    MOVE        => move, 
    ADD         => add,  
    SUB         => sub,
    BITWISE_AND => bitwise_and, 
    LOAD        => load,
    ADDM        => addm,
    SUBM        => subm,
    STORE       => store,
    JUMPU       => jumpu,
    JUMPZ       => jumpz,
    JUMPNZ      => jumpnz,  
    Z           => Z,
    ROM_EN      => ROM_EN,
    RAM_EN      => RAM_EN,
    RAM_WR      => RAM_WR,
    ADDR_SEL    => ADDR_SEL,
    DATA_SEL    => DATA_SEL,
    ALU_CTL0    => ALU_CTL0,
    ALU_CTL1    => ALU_CTL1,
    ALU_CTL2    => ALU_CTL2,
    ACC_EN      => ACC_EN,
    IR_EN       => IR_EN,
    PC_LD       => PC_LD,
    PC_EN       => PC_EN ); 

end CONTROL_LOGIC_ARCH;
