
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity DECODER is
port (
  FETCH       : in  STD_LOGIC;                 
  DECODE      : in  STD_LOGIC;                
  EXECUTE     : in  STD_LOGIC;   
  MOVE        : in  STD_LOGIC; 
  ADD         : in  STD_LOGIC;  
  SUB         : in  STD_LOGIC; 
  BITWISE_AND : in  STD_LOGIC; 
  LOAD        : in  STD_LOGIC; 
  ADDM        : in  STD_LOGIC; 
  SUBM        : in  STD_LOGIC; 
  STORE       : in  STD_LOGIC; 
  JUMPU       : in  STD_LOGIC; 
  JUMPZ       : in  STD_LOGIC; 
  JUMPNZ      : in  STD_LOGIC;  
  Z           : in  STD_LOGIC; 
  ROM_EN      : OUT STD_LOGIC;  
  RAM_EN      : OUT STD_LOGIC;  
  RAM_WR      : OUT STD_LOGIC;   
  ADDR_SEL    : OUT STD_LOGIC;  
  DATA_SEL    : OUT STD_LOGIC; 
  ALU_CTL0    : OUT STD_LOGIC; 
  ALU_CTL1    : OUT STD_LOGIC;  
  ALU_CTL2    : OUT STD_LOGIC;  
  ACC_EN      : OUT STD_LOGIC;  
  IR_EN       : OUT STD_LOGIC; 
  PC_LD       : OUT STD_LOGIC;  
  PC_EN       : OUT STD_LOGIC );  
end DECODER;

architecture DECODER_ARCH of DECODER is
begin
  process ( FETCH, DECODE, EXECUTE,
            MOVE, ADD, SUB, BITWISE_AND,
            LOAD, ADDM, SUBM, STORE,
            JUMPU, JUMPZ, JUMPNZ, Z )
  begin

    ROM_EN   <= FETCH;
    RAM_EN   <= (DECODE or EXECUTE) and (LOAD or STORE or ADDM or SUBM);
    RAM_WR   <= EXECUTE and STORE;
    ADDR_SEL <= (DECODE or EXECUTE) and (LOAD or STORE or ADDM or SUBM);
    DATA_SEL <= LOAD or ADDM or SUBM;
    ALU_CTL0 <= SUB or SUBM;
    ALU_CTL1 <= BITWISE_AND;
    ALU_CTL2 <= MOVE or LOAD;
    ACC_EN   <= (MOVE or ADD or SUB or BITWISE_AND or LOAD or ADDM or SUBM) and EXECUTE;
    IR_EN    <= FETCH;
    PC_LD    <= DECODE and (JUMPU or (JUMPZ and Z) or (JUMPNZ and not Z));
    PC_EN    <= DECODE;

  end process;
end DECODER_ARCH;
