
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity GPIO_8 is
port (
  CLK  : in  STD_LOGIC; 
  CLR  : in  STD_LOGIC; 
  WE   : in  STD_LOGIC;                
  CE   : in  STD_LOGIC;
  ADDR : in  STD_LOGIC;
  DIN  : in  STD_LOGIC_VECTOR(15 downto 0); 
  DOUT : out STD_LOGIC_VECTOR(15 downto 0);
  GPI  : in  STD_LOGIC_VECTOR(7 downto 0); 
  GPO  : out STD_LOGIC_VECTOR(7 downto 0) );
end GPIO_8;

architecture GPIO_8_ARCH of GPIO_8 is

  component REG_8
  port (
    D   : in STD_LOGIC_VECTOR(7 downto 0);         
    Q   : out STD_LOGIC_VECTOR(7 downto 0);
    CLK : in STD_LOGIC;  
    CLR : in STD_LOGIC;
    CE  : in STD_LOGIC);
  end component;
  
  component MUX_2_8 
  port (
    A, B : IN  STD_LOGIC_VECTOR(7 downto 0); 
    SEL  : IN  STD_LOGIC;                    
    Y    : OUT STD_LOGIC_VECTOR(7 downto 0));
  end component;
  
  signal en : STD_LOGIC;
  signal gpo_int : STD_LOGIC_VECTOR(7 downto 0); 
  signal dout_int : STD_LOGIC_VECTOR(7 downto 0); 

begin
  
  en <= CE AND WE;
  GPO <= gpo_int;
  DOUT <= "00000000" & dout_int;

  data_reg: REG_8 port map (
    D   => DIN(7 downto 0),      
    Q   => gpo_int,
    CLK => CLK,
    CLR => CLR,
    CE  => en ); 
	 
  data_mux : MUX_2_8 port map(
    A   => gpo_int,
	 B   => GPI,
    SEL => ADDR,                    
    Y   => dout_int );

end GPIO_8_ARCH;